@@ -33,9 +33,9 @@ __author__ = "Armand Lynch <lyncha@users.sourceforge.net>"
 __copyright__ = "Copyright 2010, Armand Lynch"
 __license__ = "LGPL"
 __url__ = "http://celljam.net/"
-__version__ = "1.0.0"
+__version__ = "1.0.1"
 
-from functools import partial
+import re
 
 from pywurfl.algorithms.wurfl import utils
 from pywurfl.algorithms.wurfl import normalizers
@@ -68,7 +68,6 @@ class AbstractMatcher(object):
         raise NotImplementedError
 
     def __call__(self, user_agent):
-        user_agent = self.normalizer(user_agent)
         devid = self.conclusive_match(user_agent)
         if not devid or devid == u"generic":
             devid = self.recovery_match(user_agent)
@@ -77,13 +76,10 @@ class AbstractMatcher(object):
         return devid
 
     def conclusive_match(self, user_agent):
+        user_agent = self.normalizer(user_agent)
         match = self.look_for_matching_user_agent(user_agent)
         #print "%s -> conclusive_match -> %s" % (user_agent, match)
-        devid = u"generic"
-        if match:
-            devid = self.user_agent_map.get(match)
-            if not devid:
-                devid = u"generic"
+        devid = self.user_agent_map.get(match, u"generic")
         return devid
 
     def look_for_matching_user_agent(self, user_agent):
@@ -96,84 +92,67 @@ class AbstractMatcher(object):
     def recovery_match(self, user_agent):
         return u"generic"
 
+    recovery_map = (
+        # Openwave
+        (u"UP.Browser/7.2", u"opwv_v72_generic"),
+        (u"UP.Browser/7", u"opwv_v7_generic"),
+        (u"UP.Browser/6.2", u"opwv_v62_generic"),
+        (u"UP.Browser/6", u"opwv_v6_generic"),
+        (u"UP.Browser/5", u"upgui_generic"),
+        (u"UP.Browser/4", u"uptext_generic"),
+        (u"UP.Browser/3", u"uptext_generic"),
+
+        # Series 60
+        (u"Series60", u"nokia_generic_series60"),
+
+        # Access/Net Front
+        (u"NetFront/3.0", u"netfront_ver3"),
+        (u"ACS-NF/3.0", u"netfront_ver3"),
+        (u"NetFront/3.1", u"netfront_ver3_1"),
+        (u"ACS-NF/3.1", u"netfront_ver3_1"),
+        (u"NetFront/3.2", u"netfront_ver3_2"),
+        (u"ACS-NF/3.2", u"netfront_ver3_2"),
+        (u"NetFront/3.3", u"netfront_ver3_3"),
+        (u"ACS-NF/3.3", u"netfront_ver3_3"),
+        (u"NetFront/3.4", u"netfront_ver3_4"),
+        (u"NetFront/3.5", u"netfront_ver3_5"),
+
+        # Windows CE
+        (u"Windows CE", u"ms_mobile_browser_ver1"),
+
+        # web browsers?
+        (u"Mozilla/4.0", u"generic_web_browser"),
+        (u"Mozilla/5.0", u"generic_web_browser"),
+        (u"Mozilla/5.0", u"generic_web_browser"),
+
+        # Generic XHTML
+        (u"Mozilla/", u"generic_xhtml"),
+        (u"ObigoInternetBrowser/Q03C", u"generic_xhtml"),
+        (u"AU-MIC/2", u"generic_xhtml"),
+        (u"AU-MIC-", u"generic_xhtml"),
+        (u"AU-OBIGO/", u"generic_xhtml"),
+        (u"Obigo/Q03", u"generic_xhtml"),
+        (u"Obigo/Q04", u"generic_xhtml"),
+        (u"ObigoInternetBrowser/2", u"generic_xhtml"),
+        (u"Teleca Q03B1", u"generic_xhtml"),
+
+        # Opera Mini
+        (u"Opera Mini/1", u"opera_mini_ver1"),
+        (u"Opera Mini/2", u"opera_mini_ver2"),
+        (u"Opera Mini/3", u"opera_mini_ver3"),
+        (u"Opera Mini/4", u"opera_mini_ver4"),
+
+        # DoCoMo
+        (u"DoCoMo", u"docomo_generic_jap_ver1"),
+        (u"KDDI", u"docomo_generic_jap_ver1"))
+
     def catch_all_recovery_match(self, user_agent):
 
         match = u"generic"
-
-        # Openwave
-        if u"UP.Browser/7.2" in user_agent:
-            match = u"opwv_v72_generic"
-        elif u"UP.Browser/7" in user_agent:
-            match = u"opwv_v7_generic"
-        elif u"UP.Browser/6.2" in user_agent:
-            match = u"opwv_v62_generic"
-        elif u"UP.Browser/6" in user_agent:
-            match = u"opwv_v6_generic"
-        elif u"UP.Browser/5" in user_agent:
-            match = u"upgui_generic"
-        elif u"UP.Browser/4" in user_agent:
-            match = u"uptext_generic"
-        elif u"UP.Browser/3" in user_agent:
-            match = u"uptext_generic"
-
-        # Series 60
-        elif u"Series60" in user_agent:
-            match = u"nokia_generic_series60"
-
-        # Access/Net Front
-        elif u"NetFront/3.0" in user_agent or u"ACS-NF/3.0" in user_agent:
-            match = u"netfront_ver3"
-        elif u"NetFront/3.1" in user_agent or u"ACS-NF/3.1" in user_agent:
-            match = u"netfront_ver3_1"
-        elif u"NetFront/3.2" in user_agent or u"ACS-NF/3.2" in user_agent:
-            match = u"netfront_ver3_2"
-        elif u"NetFront/3.3" in user_agent or u"ACS-NF/3.3" in user_agent:
-            match = u"netfront_ver3_3"
-        elif u"NetFront/3.4" in user_agent:
-            match = u"netfront_ver3_4"
-        elif u"NetFront/3.5" in user_agent:
-            match = u"netfront_ver3_5"
-
-        # Windows CE
-        elif u"Windows CE" in user_agent:
-            match = u"ms_mobile_browser_ver1"
-
-        # web browsers?
-        elif u"Mozilla/4.0" in user_agent:
-            match = u"generic_web_browser"
-        elif u"Mozilla/5.0" in user_agent:
-            match = u"generic_web_browser"
-        elif u"Mozilla/6.0" in user_agent:
-            match = u"generic_web_browser"
-
-        # Generic XHTML
-        elif u"Mozilla/" in user_agent:
-            match = u"generic_xhtml"
-
-        elif (u"ObigoInternetBrowser/Q03C" in user_agent or
-              u"AU-MIC/2" in user_agent or
-              u"AU-MIC-" in user_agent or
-              u"AU-OBIGO/" in user_agent or
-              u"Obigo/Q03" in user_agent or
-              u"Obigo/Q04" in user_agent or
-              u"ObigoInternetBrowser/2" in user_agent or
-              u"Teleca Q03B1" in user_agent):
-            match = u"generic_xhtml"
-
-        # Opera Mini
-        elif u"Opera Mini/1" in user_agent:
-            match = u"opera_mini_ver1"
-        elif u"Opera Mini/2" in user_agent:
-            match = u"opera_mini_ver2"
-        elif u"Opera Mini/3" in user_agent:
-            match = u"opera_mini_ver3"
-        elif u"Opera Mini/4" in user_agent:
-            match = u"opera_mini_ver4"
-
-        # DoCoMo
-        elif user_agent.startswith(u"DoCoMo") or user_agent.startswith(u"KDDI"):
-            match = u"docomo_generic_jap_ver1"
-
+        for partial_agent, wdevice in self.recovery_map:
+            if partial_agent in user_agent:
+                match = wdevice
+                break
         return match
 
 
@@ -188,11 +167,15 @@ class AndroidMatcher(AbstractMatcher):
         return u"Android" in user_agent
 
     def look_for_matching_user_agent(self, user_agent):
-        tolerance = uiol(user_agent, u" ", uiol(user_agent, u"Android"))
+        tolerance = uiol(user_agent, u" ",
+                         start_index=uiol(user_agent, u"Android"))
         match = ris_match(self.user_agents, user_agent, tolerance)
         #print "AndroidMatcher %s -> l_f_m_ua -> %s" % (user_agent, match)
         return match
 
+    def recovery_match(self, user_agent):
+        return u"generic_android"
+
 
 class AOLMatcher(AbstractMatcher):
     def can_handle(self, user_agent):
@@ -200,61 +183,67 @@ class AOLMatcher(AbstractMatcher):
 
 
 class AppleMatcher(AbstractMatcher):
-    APPLE_LD_TOLLERANCE = 5
+    APPLE_LD_TOLERANCE = 5
 
     def can_handle(self, user_agent):
-        return u"iPhone" in user_agent or u"iPod" in user_agent
+        return (u"iPhone" in user_agent or u"iPod" in user_agent or u"iPad" in
+                user_agent)
 
     def look_for_matching_user_agent(self, user_agent):
-        match = ld_match(self.user_agents, user_agent, self.APPLE_LD_TOLLERANCE)
+        tolerance = utils.first_semi_colon(user_agent)
+        match = ld_match(self.user_agents, user_agent, tolerance)
         #print "AppleMatcher %s -> l_f_m_ua -> %s" % (user_agent, match)
         return match
 
     def recovery_match(self, user_agent):
-        if u"iPhone" in user_agent:
-            return u"apple_iphone_ver1"
-        return u"apple_ipod_touch_ver1"
+        if u"iPad" in user_agent:
+            return "apple_ipad_ver1"
+        if u"iPod" in user_agent:
+            return u"apple_ipod_ver1"
+        return u"apple_iphone_ver1"
 
 
 class BenQMatcher(AbstractMatcher):
     def can_handle(self, user_agent):
-        return user_agent.startswith(u"BenQ") or user_agent.startswith(u"BENQ")
+        return user_agent.startswith(u"BENQ") or user_agent.startswith(u"BenQ")
 
 
 class BlackberryMatcher(AbstractMatcher):
+    blackberries = {}
+    blackberries["2."] = u"blackberry_generic_ver2"
+    blackberries["3.2"] = u"blackberry_generic_ver3_sub2"
+    blackberries["3.3"] = u"blackberry_generic_ver3_sub30"
+    blackberries["3.5"] = u"blackberry_generic_ver3_sub50"
+    blackberries["3.6"] = u"blackberry_generic_ver3_sub60"
+    blackberries["3.7"] = u"blackberry_generic_ver3_sub70"
+    blackberries["4."] = u"blackberry_generic_ver4"
+
     def can_handle(self, user_agent):
         return u"BlackBerry" in user_agent
 
     def recovery_match(self, user_agent):
         match = u"generic"
 
-        if user_agent.startswith("BlackBerry"):
-            try:
-                position = user_agent.index('/')
-                if position + 4 < len(user_agent):
-                    version = user_agent[position+1:position+4]
-                    if version.startswith(u"2."):
-                        match = u"blackberry_generic_ver2"
-                    elif version.startswith(u"3.2"):
-                        match = u"blackberry_generic_ver3_sub2"
-                    elif version.startswith(u"3.3"):
-                        match = u"blackberry_generic_ver3_sub30"
-                    elif version.startswith(u"3.5"):
-                        match = u"blackberry_generic_ver3_sub50"
-                    elif version.startswith(u"3.6"):
-                        match = u"blackberry_generic_ver3_sub60"
-                    elif version.startswith(u"3.7"):
-                        match = u"blackberry_generic_ver3_sub70"
-                    elif version.startswith(u"4."):
-                        match = u"blackberry_generic_ver4"
-            except ValueError:
-                pass
+        if user_agent.startswith(u"BlackBerry"):
+            version = self.get_version(user_agent)
+            match = self.blackberries.get(version, u"generic")
 
         return match
 
+    def get_version(self, user_agent):
+        version = u""
+        position = user_agent.index('/')
+        if position + 4 < len(user_agent):
+            version = user_agent[position+1:position+4]
+        return version
+
 
 class BotMatcher(AbstractMatcher):
-    bots = (u"bot", u"crawler", u"novarra", u"transcoder")
+    bots = (u"bot", u"crawler", u"spider", u"novarra", u"transcoder",
+            u"yahoo! searchmonkey", u"yahoo! slurp", u"feedfetcher-google",
+            u"toolbar", u"mowser")
+
+    BOT_TOLERANCE = 4
 
     def can_handle(self, user_agent):
         user_agent = user_agent.lower()
@@ -263,9 +252,16 @@ class BotMatcher(AbstractMatcher):
                 return True
         return False
 
+    def look_for_matching_user_agent(self, user_agent):
+        match = ld_match(self.user_agents, user_agent, self.BOT_TOLERANCE)
+        return match
+
+    def recovery_match(self, user_agent):
+        return u"generic_web_crawler"
+
 
 class CatchAllMatcher(AbstractMatcher):
-    MOZILLA_LD_TOLLERANCE = 4
+    MOZILLA_LD_TOLERANCE = 4
 
     def can_handle(self, user_agent):
         return True
@@ -274,13 +270,13 @@ class CatchAllMatcher(AbstractMatcher):
         if user_agent.startswith(u"Mozilla"):
             if user_agent.startswith(u"Mozilla/4"):
                 match = ld_match(self.extract_uas(u"Mozilla/4"), user_agent,
-                                 self.MOZILLA_LD_TOLLERANCE)
+                                 self.MOZILLA_LD_TOLERANCE)
             elif user_agent.startswith(u"Mozilla/5"):
                 match = ld_match(self.extract_uas(u"Mozilla/5"), user_agent,
-                                 self.MOZILLA_LD_TOLLERANCE)
+                                 self.MOZILLA_LD_TOLERANCE)
             else:
                 match = ld_match(self.extract_uas(u"Mozilla"), user_agent,
-                                 self.MOZILLA_LD_TOLLERANCE)
+                                 self.MOZILLA_LD_TOLERANCE)
         else:
             match = AbstractMatcher.look_for_matching_user_agent(self,
                                                                  user_agent)
@@ -288,7 +284,7 @@ class CatchAllMatcher(AbstractMatcher):
         return match
 
     def extract_uas(self, start):
-        return [x for x in self.user_agents if x.startswith(start)]
+        return (x for x in self.user_agents if x.startswith(start))
 
 
 class ChromeMatcher(AbstractMatcher):
@@ -305,6 +301,8 @@ class DoCoMoMatcher(AbstractMatcher):
         return u""
 
     def recovery_match(self, user_agent):
+        if user_agent.startswith(u"DoCoMo/2"):
+            return u"docomo_generic_jap_ver2"
         return u"docomo_generic_jap_ver1"
 
 
@@ -327,19 +325,23 @@ class HTCMatcher(AbstractMatcher):
 
 class KDDIMatcher(AbstractMatcher):
     def can_handle(self, user_agent):
-        return user_agent.startswith(u"KDDI")
+        return u"KDDI" in user_agent
 
     def look_for_matching_user_agent(self, user_agent):
-        if user_agent.startswith("KDDI/"):
+        if user_agent.startswith(u"KDDI/"):
             tolerance = utils.second_slash(user_agent)
-            match = ris_match(self.user_agents, user_agent, tolerance)
+        elif user_agent.startswith(u"KDDI"):
+            tolerance = utils.first_slash(user_agent)
         else:
-            match = AbstractMatcher.look_for_matching_user_agent(self,
-                                                                 user_agent)
+            tolerance = uiol(user_agent, ")")
+
+        match = ris_match(self.user_agents, user_agent, tolerance)
         #print "KDDIMatcher %s -> l_f_m_ua -> %s" % (user_agent, match)
         return match
 
     def recovery_match(self, user_agent):
+        if u"Opera" in user_agent:
+            return u"opera"
         return u"opwv_v62_generic"
 
 
@@ -358,17 +360,18 @@ class KyoceraMatcher(AbstractMatcher):
 
 class LGMatcher(AbstractMatcher):
     def can_handle(self, user_agent):
-        return user_agent.startswith(u"LG") or user_agent.startswith(u"lg")
+        return (user_agent.startswith(u"lg") or u"LG-" in user_agent or
+                u"LGE" in user_agent)
 
     def look_for_matching_user_agent(self, user_agent):
-        if user_agent.startswith(u"LGE/") or user_agent.startswith("LG/"):
+        if u"Vodafone" in user_agent:
+            tolerance = uiol(user_agent, u"LG")
+        elif user_agent.startswith(u"LGE/") or user_agent.startswith("LG/"):
             tolerance = utils.second_slash(user_agent)
-            match = ris_match(self.user_agents, user_agent, tolerance)
         else:
-            match = AbstractMatcher.look_for_matching_user_agent(self,
-                                                                 user_agent)
-        #print "LGMatcher %s -> l_f_m_ua -> %s" % (user_agent, match)
-        return match
+            tolerance = utils.first_slash(user_agent)
+
+        return ris_match(self.user_agents, user_agent, tolerance)
 
 
 class MitsubishiMatcher(AbstractMatcher):
@@ -433,7 +436,8 @@ class NokiaMatcher(AbstractMatcher):
         return u"Nokia" in user_agent
 
     def look_for_matching_user_agent(self, user_agent):
-        tolerance = uiol(user_agent, u"/", user_agent.index(u"Nokia"))
+        tolerance = uiol(user_agent, u"/",
+                         start_index=user_agent.index(u"Nokia"))
         #print "NokiaMatcher tolerance %s" % tolerance
         match = ris_match(self.user_agents, user_agent, tolerance)
         #print "NokiaMatcher %s -> l_f_m_ua -> %s" % (user_agent, match)
@@ -446,11 +450,20 @@ class NokiaMatcher(AbstractMatcher):
             match = u"nokia_generic_series80"
         else:
             match = u"generic"
+        return match
 
 
 class OperaMatcher(AbstractMatcher):
 
-    OPERA_TOLERANCE = 5
+    OPERA_TOLERANCE = 3
+
+    operas = {}
+    operas["7"] = u"opera_7"
+    operas["8"] = u"opera_8"
+    operas["9"] = u"opera_9"
+    operas["10"] = u"opera_10"
+
+    opera_re = re.compile(r".*Opera/(\d+).*")
 
     def can_handle(self, user_agent):
         return (not utils.is_mobile_browser(user_agent) and
@@ -461,6 +474,12 @@ class OperaMatcher(AbstractMatcher):
         #print "OperaMatcher %s -> l_f_m_ua -> %s" % (user_agent, match)
         return match
 
+    def recovery_match(self, user_agent):
+        match = self.opera_re.match(user_agent)
+        if match:
+            return self.operas.get(match.groups()[0], u"opera")
+        return u"opera"
+
 
 class OperaMiniMatcher(AbstractMatcher):
     def can_handle(self, user_agent):
@@ -476,7 +495,6 @@ class OperaMiniMatcher(AbstractMatcher):
             match = u"opera_mini_ver3"
         elif u"Opera Mini/4" in user_agent:
             match = u"opera_mini_ver4"
-
         return match
 
 
@@ -539,7 +557,6 @@ class SafariMatcher(AbstractMatcher):
         return match
 
 
-
 class SagemMatcher(AbstractMatcher):
     def can_handle(self, user_agent):
         return (user_agent.startswith(u"Sagem") or
@@ -557,24 +574,21 @@ class SamsungMatcher(AbstractMatcher):
                 user_agent.startswith(u"SCH"))
 
     def look_for_matching_user_agent(self, user_agent):
-        match = u""
         if (user_agent.startswith(u"SEC-") or
             user_agent.startswith(u"SAMSUNG-") or
             user_agent.startswith(u"SCH")):
-            match = AbstractMatcher.look_for_matching_user_agent(self,
-                                                                 user_agent)
+            tolerance = utils.first_slash(user_agent)
         elif (user_agent.startswith(u"Samsung") or
               user_agent.startswith(u"SPH") or
               user_agent.startswith(u"SGH")):
             tolerance = utils.first_space(user_agent)
-            match = ris_match(self.user_agents, user_agent,
-                                         tolerance)
         elif user_agent.startswith(u"SAMSUNG/"):
             tolerance = utils.second_slash(user_agent)
-            match = ris_match(self.user_agents, user_agent, tolerance)
         elif u"Samsung/SGH-L870" in user_agent:
             tolerance = uiol(user_agent, u"/", 5)
-            match = ris_match(self.user_agents, user_agent, tolerance)
+        else:
+            tolerance = len(user_agent)
+        match = ris_match(self.user_agents, user_agent, tolerance)
         #print "SamsungMatcher %s -> l_f_m_ua -> %s" % (user_agent, match)
         return match
 
@@ -616,7 +630,7 @@ class SPVMatcher(AbstractMatcher):
         return u"SPV" in user_agent
 
     def look_for_matching_user_agent(self, user_agent):
-        tolerance = uiol(user_agent, u";", uiol(user_agent, u"SPV"))
+        tolerance = uiol(user_agent, u";", start_index=uiol(user_agent, u"SPV"))
         match = ris_match(self.user_agents, user_agent, tolerance)
         return match
 
@@ -628,19 +642,12 @@ class ToshibaMatcher(AbstractMatcher):
 
 class VodafoneMatcher(AbstractMatcher):
 
-    VODAFONE_TOLLERANCE = 3
-
     def can_handle(self, user_agent):
         return user_agent.startswith(u"Vodafone")
 
     def look_for_matching_user_agent(self, user_agent):
-        tolerance = self.VODAFONE_TOLLERANCE
-        if u"Nokia" in user_agent:
-            tolerance = uiol(user_agent, u"/", uiol(user_agent, u"Nokia"))
-            match = ris_match(self.user_agents, user_agent, tolerance)
-        else:
-            match = ld_match(self.user_agents, user_agent,
-                             self.VODAFONE_TOLLERANCE)
+        tolerance = uiol(user_agent, u"/", 3)
+        match = ris_match(self.user_agents, user_agent, tolerance)
         #print "VodafoneMatcher %s -> l_f_m_ua -> %s" % (user_agent, match)
         return match
 
@@ -649,7 +656,8 @@ class WindowsCEMatcher(AbstractMatcher):
     WINDOWS_CE_TOLERANCE = 3
 
     def can_handle(self, user_agent):
-        return u"Mozilla/" in user_agent and u"Windows CE" in user_agent
+        return u"Mozilla/" in user_agent and (u"Windows CE" in user_agent or
+                                              u"WindowsCE" in user_agent)
 
     def look_for_matching_user_agent(self, user_agent):
         match = ld_match(self.user_agents, user_agent,
@@ -660,8 +668,8 @@ class WindowsCEMatcher(AbstractMatcher):
         return u"ms_mobile_browser_ver1"
 
 
-handlers = [VodafoneMatcher(),
-            NokiaMatcher(),
+handlers = [NokiaMatcher(),
+            AndroidMatcher(normalizers.android),
             SonyEricssonMatcher(),
             MotorolaMatcher(),
             BlackberryMatcher(),
@@ -684,14 +692,14 @@ handlers = [VodafoneMatcher(),
             ToshibaMatcher(),
             GrundigMatcher(),
             HTCMatcher(),
+            BotMatcher(),
             SPVMatcher(),
             WindowsCEMatcher(),
             PortalmmmMatcher(),
             DoCoMoMatcher(),
             KDDIMatcher(),
-            AndroidMatcher(),
+            VodafoneMatcher(),
             OperaMiniMatcher(),
-            BotMatcher(),
             ChromeMatcher(normalizers.chrome),
             AOLMatcher(),
             OperaMatcher(normalizers.opera),
