@@ -1,5 +1,5 @@
 # WURFL Processor - Wireless Universal Resource File Processor in Python
-# Copyright (C) 2004-2009 Armand Lynch
+# Copyright (C) 2004-2010 Armand Lynch
 #
 # This library is free software; you can redistribute it and/or modify it
 # under the terms of the GNU Lesser General Public License as published by the
@@ -39,7 +39,7 @@ from pywurfl.exceptions import WURFLException
 
 __author__ = "Armand Lynch <lyncha@users.sourceforge.net>"
 __contributors__ = "Pau Aliagas <pau@newtral.org>"
-__copyright__ = "Copyright 2004-2009, Armand Lynch"
+__copyright__ = "Copyright 2004-2010, Armand Lynch"
 __license__ = "LGPL"
 __url__ = "http://celljam.net/"
 __all__ = ['DeferredDeviceError', 'DeviceHandler', 'WurflProcessor', 'main',
@@ -68,11 +68,11 @@ class DeviceHandler(object):
         @type device: elementtree.Element
         """
 
-        self.devua = device.attrib["user_agent"]
-        self.devid = device.attrib["id"]
-        self.parent = device.attrib["fall_back"]
-        if ("actual_device_root" in device.attrib and
-            device.attrib['actual_device_root'].lower() == "true"):
+        self.devua = device.attrib[u"user_agent"]
+        self.devid = device.attrib[u"id"]
+        self.parent = device.attrib[u"fall_back"]
+        if (u"actual_device_root" in device.attrib and
+            device.attrib[u"actual_device_root"].lower() == u"true"):
             self.actual_device_root = True
         else:
             self.actual_device_root = False
@@ -104,10 +104,10 @@ class WurflProcessor(object):
         elif wurflxml.endswith(".zip"):
             from zipfile import ZipFile
             from cStringIO import StringIO
-            zipfile = ZipFile(wurflxml)
-            file_handle = StringIO(zipfile.read(zipfile.namelist()[0]))
+            zfile = ZipFile(wurflxml)
+            file_handle = StringIO(zfile.read(zfile.namelist()[0]))
         else:
-            file_handle = file(wurflxml,"rb")
+            file_handle = open(wurflxml, "rb")
         self.file_handle = file_handle
 
         self.tree = parse(self.file_handle)
